@@ -31,7 +31,7 @@
   SUCH DAMAGE.
 */
 
-#define TARGET_API_MAC_CARBON 1
+#define TARGET_API_MAC_CARBON  1
 
 #include <Carbon/Carbon.h>
 
@@ -50,6 +50,11 @@
 
 #define ENABLE_HELPER  1
 
+//#define DEBUG_KEYEVENT  1
+//#define DEBUG_MENU  1
+//#define DEBUG_CANDIDATES  1
+//#define DEBUG_SESSION  1
+
 
 const short kMENU_Pencil = kBaseResourceID + 1;
 
@@ -101,14 +106,26 @@ static CFStringRef gHideKeyboardPaletteMenuItemString;
 static CFStringRef gShowSendEventPaletteMenuItemString;
 static CFStringRef gHideSendEventPaletteMenuItemString;
 
-static pascal OSStatus MUIMPencilMenuEventHandler(EventHandlerCallRef
-                                                 inEventHandlerCallRef,
-                                                 EventRef inEventRef,
-                                                 void *inUserData);
+static pascal OSStatus
+MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
+                           EventRef inEventRef, void *inUserData);
 
-static int ConvertKeyVal(int inKey);
+static void
+CreateUIMContext(CFStringRef inIMName);
 
-static int ConvertModifier(int inMod);
+static void
+ReleaseUIMContext();
+
+static void
+NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver, 
+                     CFStringRef inName, const void *inObject, 
+                     CFDictionaryRef inUserInfo);
+
+static int
+ConvertKeyVal(int inKey);
+
+static int
+ConvertModifier(int inMod);
 
 
 ComponentResult
@@ -222,7 +239,9 @@ MUIMInitialize(ComponentInstance inComponentInstance,
     result = MUIMInitializeMessageReceiving();
   */
 
+#ifdef DEBUG_SESSION
   DEBUG_PRINT("MUIMInitialize()\n");
+#endif
 
   {
     char *home = getenv("HOME");
@@ -246,25 +265,20 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   gUimSock = NULL;
   gUimRun = NULL;
 
+  CreateUIMContext(NULL);
+
   {
-    const char *imName;
-    CFPropertyListRef imVal;
-    CFStringRef appID = CFSTR(kAppID);
+    CFNotificationCenterRef center =
+      CFNotificationCenterGetDistributedCenter();
 
-    // Load the preferences
-    imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), appID);
-    if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
-      imName = CFStringGetCStringPtr((CFStringRef) imVal, kCFStringEncodingASCII);
-    else
-      imName = kDefaultIM;
-
-    DEBUG_PRINT("MUIMSessionOpen() imName='%s'\n", imName);
-
-    gUC = uim_create_context(gSessionList, "UTF-8", NULL, imName,
-                             uim_iconv, UIMCommitString);
-    if (!gUC) {
-      DEBUG_PRINT("uim_create_context() failed\n");
-      result = memFullErr;
+    if (center) {
+      CFNotificationCenterAddObserver(center, NULL, NotificationCallback,
+                                      NULL, CFSTR(kAppID),
+                                      //CFNotificationSuspensionBehaviorDrop
+                                      CFNotificationSuspensionBehaviorCoalesce
+                                      //CFNotificationSuspensionBehaviorHold
+                                      //CFNotificationSuspensionBehaviorDeliverImmediately
+                                      );
     }
   }
 
@@ -274,7 +288,9 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 void
 MUIMTerminate(ComponentInstance inComponentInstance)
 {
+#ifdef DEBUG_SESSION
     DEBUG_PRINT("MUIMTerminate()\n");
+#endif
 
 #ifdef ENABLE_HELPER
     UIMHelperClose();
@@ -298,10 +314,14 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
   if (*outSessionHandle == nil) {
     *outSessionHandle =
       (MUIMSessionHandle) NewHandle(sizeof(MUIMSessionRecord));
+#ifdef DEBUG_SESSION
     //DEBUG_PRINT("MUIMSessionOpen() NewHandle=%p\n", *outSessionHandle);
+#endif
   }
 
   if (*outSessionHandle) {
+    gNumSession++;
+
     if (!gSessionList)
       gSessionList = (MUIMSessionHandle *)
         malloc(sizeof(MUIMSessionHandle) * gNumSession);
@@ -309,8 +329,7 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
       gSessionList = (MUIMSessionHandle *)
         realloc(gSessionList, sizeof(MUIMSessionHandle) * gNumSession);
 
-    gSessionList[gNumSession] = *outSessionHandle;
-    gNumSession++;
+    gSessionList[gNumSession - 1] = *outSessionHandle;
 
     (**outSessionHandle)->fComponentInstance = inComponentInstance;
     (**outSessionHandle)->fLastUpdateLength = 0;
@@ -319,29 +338,6 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
 
     //(**outSessionHandle)->fUC = gUC;
 
-#ifdef ENABLE_HELPER
-    UIMCheckHelper(*outSessionHandle);
-#endif
-
-    uim_set_preedit_cb(gUC,
-                       UIMPreeditClear,
-                       UIMPreeditPushback,
-                       UIMPreeditUpdate);
-#ifdef ENABLE_HELPER
-    uim_set_prop_list_update_cb(gUC,
-                                UIMUpdatePropList);
-    uim_set_prop_label_update_cb(gUC,
-                                 UIMUpdatePropLabel);
-
-    uim_prop_list_update(gUC);
-    uim_prop_label_update(gUC);
-#endif
-    uim_set_candidate_selector_cb(gUC,
-                                  UIMCandAcivate,
-                                  UIMCandSelect,
-                                  UIMCandShiftPage,
-                                  UIMCandDeactivate);
-
     (**outSessionHandle)->fLastUpdateLength = 0;
     (**outSessionHandle)->fFixBuffer = NULL;
     (**outSessionHandle)->fFixLen = 0;
@@ -373,17 +369,21 @@ MUIMSessionClose(MUIMSessionHandle inSessionHandle)
 {
   UInt32 i, j;
 
+#ifdef DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionClose() inSessionHandle=%p\n",
               inSessionHandle);
+#endif
 
   for (i = 0; i < gNumSession; i++) {
     if (gSessionList[i] == inSessionHandle) {
       gSessionList[i] = NULL;
       for (j = i; j < gNumSession - 1; j++)
         gSessionList[j] = gSessionList[j + 1];
+      break;
     }
   }
-  gNumSession--;
+  if (i < gNumSession)
+    gNumSession--;
 
   if (inSessionHandle) {
     for (i = 0; i < (*inSessionHandle)->fSegmentCount; i++)
@@ -409,15 +409,15 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
                                       smScriptKeys);
   SetScriptVariable (smJapanese, smScriptKeys, keyboardID);
 
+#ifdef DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
               gActiveSession);
+#endif
 
 #ifdef ENABLE_HELPER
-  UIMCheckHelper(inSessionHandle);
-
+  UIMCheckHelper();
   uim_prop_list_update(gUC);
   uim_prop_label_update(gUC);
-
   uim_helper_client_focus_in(gUC);
 #endif
 
@@ -427,8 +427,10 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 ComponentResult
 MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
 {
+#ifdef DEBUG_SESSION
   DEBUG_PRINT("MUIMSessionDeactivate() gActiveSession=%p\n",
               gActiveSession);
+#endif
 
 #ifdef ENABLE_HELPER
   uim_helper_client_focus_out(gUC);
@@ -488,8 +490,10 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
     GetEventParameter(inEventRef, kEventParamKeyModifiers, typeUInt32, nil,
                       sizeof(modifiers), nil, &modifiers);
 
+#ifdef DEBUG_KEYEVENT
     DEBUG_PRINT("MUIMSessionEvent() keycode=0x%lx, char=%c, charCode=0x%x, modifiers=0x%lx\n",
                 keyCode, charCode, charCode, modifiers);
+#endif
 
     if (!(modifiers & cmdKey))
        handled = MUIMHandleInput(inSessionHandle, keyCode, charCode, modifiers);
@@ -498,24 +502,14 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   return handled;
 }
 
-/*
-ComponentResult
-MUIMSessionFix(MUIMSessionHandle inSessionHandle)
-{
-  ComponentResult result = noErr;
-
-  DEBUG_PRINT("MUIMSessionFix()\n");
-
-  return result;
-}
-*/
-
 #pragma mark -
 
 MUIMSessionHandle
 MUIMGetActiveSession(void)
 {
+#ifdef DEBUG_SESSION
   DEBUG_PRINT("MUIMGetActiveSession()\n");
+#endif
 
   return gActiveSession;
 }
@@ -533,8 +527,10 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   handled = FALSE;
   isScriptKey = FALSE;
 
+#ifdef DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() inCharCode=%02x inModifiers=%lx\n",
               inCharCode, inModifiers);
+#endif
 
   /* Check for special keys first */
   for (i = 0; KeycodeToUKey[i].ukey; i++) {
@@ -552,7 +548,9 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 
   mod = ConvertModifier(inModifiers);
 
+#ifdef DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
+#endif
 
   rv = uim_press_key(gUC, key, mod);
 
@@ -561,8 +559,10 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
   if (!rv || isScriptKey)
     handled = TRUE;
 
+#ifdef DEBUG_KEYEVENT
   DEBUG_PRINT("MUIMHandleInput() uim_press_key handled=%s\n",
               handled ? "true" : "false");
+#endif
 
   return handled;
 }
@@ -570,8 +570,10 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 void
 MUIMUpdateShowHideKeyboardPaletteMenuItem(Boolean inIsHidden)
 {
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("MUIMUpdateShowHideKeyboardPaletteMenuItem() inIsHidden=%d\n",
               inIsHidden);
+#endif
 
   if (inIsHidden)
     SetMenuItemTextWithCFString(gPencilMenu,
@@ -586,8 +588,10 @@ MUIMUpdateShowHideKeyboardPaletteMenuItem(Boolean inIsHidden)
 void
 MUIMUpdateShowHideSendEventPaletteMenuItem(Boolean inIsHidden)
 {
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("MUIMUpdateShowHideSendEventPaletteMenuItem() inIsHidden=%d\n",
               inIsHidden);
+#endif
 
   if (inIsHidden)
     SetMenuItemTextWithCFString(gPencilMenu,
@@ -608,7 +612,9 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
   OSStatus result;
   HICommand command;
 
+#ifdef DEBUG_MENU
   DEBUG_PRINT("MUIMPencilMenuEventHandler()\n");
+#endif
 
   result =
     GetEventParameter(inEventRef, kEventParamDirectObject, typeHICommand,
@@ -638,10 +644,98 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
   return result;
 }
 
+static void
+CreateUIMContext(CFStringRef inIMName)
+{
+  const char *imName;
+
+  if (inIMName) {
+    imName = CFStringGetCStringPtr(inIMName, kCFStringEncodingASCII);
+  }
+  else {
+    CFPropertyListRef imVal;
+
+    // Load the preferences
+    imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), CFSTR(kAppID));
+    if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
+      imName = CFStringGetCStringPtr((CFStringRef) imVal,
+                                     kCFStringEncodingASCII);
+    else
+      imName = kDefaultIM;
+  }
+
+  //DEBUG_PRINT("CreateUIMContext() imName='%s'\n", imName);
+
+  gUC = uim_create_context(gSessionList, "UTF-8", NULL, imName,
+                           uim_iconv, UIMCommitString);
+
+  if (!gUC)
+    return;
+
+  uim_set_preedit_cb(gUC,
+                     UIMPreeditClear,
+                     UIMPreeditPushback,
+                     UIMPreeditUpdate);
+
+#ifdef ENABLE_HELPER
+  UIMCheckHelper();
+  uim_set_prop_list_update_cb(gUC, UIMUpdatePropList);
+  uim_set_prop_label_update_cb(gUC, UIMUpdatePropLabel);
+  uim_prop_list_update(gUC);
+  uim_prop_label_update(gUC);
+#endif
+
+  uim_set_candidate_selector_cb(gUC,
+                                UIMCandAcivate,
+                                UIMCandSelect,
+                                UIMCandShiftPage,
+                                UIMCandDeactivate);
+}
+
+static void
+ReleaseUIMContext()
+{
+  uim_release_context(gUC);
+  gUC = NULL;
+}
+
+static void
+NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver, 
+                     CFStringRef inName, const void *inObject, 
+                     CFDictionaryRef inUserInfo)
+{
+  CFStringRef *keys;
+  CFStringRef *values;
+  int count;
+
+  count = CFDictionaryGetCount(inUserInfo);
+  //DEBUG_PRINT("NotificationCallback() count=%d\n", count);
+
+  if (count > 0) {
+    keys = (CFStringRef *) malloc(count * sizeof(CFStringRef));
+    values = (CFStringRef *) malloc(count * sizeof(CFStringRef));
+
+    CFDictionaryGetKeysAndValues(inUserInfo,
+                                 (const void **) keys, (const void **) values);
+
+    //DEBUG_PRINT("NotificationCallback() keys[0]='%s' values[0]='%s'\n",
+    //            CFStringGetCStringPtr(keys[0], kCFStringEncodingASCII),
+    //            CFStringGetCStringPtr(values[0], kCFStringEncodingASCII));
+
+    ReleaseUIMContext();
+    CreateUIMContext(values[0]);
+
+    free(keys);
+    free(values);
+  }
+}
+
 static int
 ConvertKeyVal(int inKey)
 {
+#ifdef DEBUG_KEYEVENT
   DEBUG_PRINT("ConvertKeyVal() inKey=%02x\n", inKey);
+#endif
 
   if (inKey >= 0x01 && inKey <= 0x1a)
     return inKey + 0x60;
@@ -653,7 +747,9 @@ ConvertModifier(int inMod)
 {
   int modifier = 0;
 
+#ifdef DEBUG_KEYEVENT
   DEBUG_PRINT("ConvertModifier() inMod=%lx\n", inMod);
+#endif
 
   if (inMod & shiftKey)
     modifier += UMod_Shift;
