@@ -43,7 +43,7 @@ static MacUIMHelperExtra *sharedExtra;
 static void
 uimDisconnect();
 
-static char *
+static NSString *
 convertHelperString(char *str);
 
 
@@ -358,7 +358,7 @@ convertHelperString(char *str);
 
   //printf("MacUIMHelperExtra::helperParse: str='%s'\n", str);
 
-  nsstr = [[NSString alloc] initWithUTF8String:convertHelperString(str)];
+  nsstr = convertHelperString(str);
 
   if (nsstr) {
     array = [nsstr componentsSeparatedByString:@"\n"];
@@ -436,7 +436,8 @@ convertHelperString(char *str);
     if (cols && [cols count] >= 3) {
       col = [cols objectAtIndex:0];
       if ([col compare:@"branch"] == NSOrderedSame) {
-        NSMutableString *branch = [[NSMutableString alloc] initWithString:[cols objectAtIndex:1]];
+        NSMutableString *branch =
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:1]];
         /*
         printf("MacUIMHelperExtra::propListUpdate: branch='%s'\n",
                [branch UTF8String]);
@@ -444,8 +445,10 @@ convertHelperString(char *str);
         [labels addObject:branch];
       }
       else if ([col compare:@"leaf"] == NSOrderedSame) {
-        NSMutableString *mode = [[NSMutableString alloc] initWithString:[cols objectAtIndex:2]];
-        NSMutableString *prop = [[NSMutableString alloc] initWithString:[cols objectAtIndex:4]];
+        NSMutableString *mode =
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:2]];
+        NSMutableString *prop =
+          [[NSMutableString alloc] initWithString:[cols objectAtIndex:4]];
         /*
         printf("MacUIMHelperExtra::propListUpdate: leaf='%s'\n",
                [mode UTF8String]);
@@ -519,54 +522,36 @@ uimDisconnect()
 }
 
 
-static char *
+static NSString *
 convertHelperString(char *str)
 {
   char *line, *tmp;
   char *charset = NULL;
-  iconv_t cd;
-  size_t outbufsize = BUFSIZ;
-  char *inbuf, *outbuf, *convstr = NULL;
-  const char *inchar;
-  char *outchar;
-  size_t inbytesleft, outbytesleft;
-  size_t ret_val;
+  NSString *convstr;
 
   line = strdup(str);
   if (tmp = strstr(line, "charset=")) {
-    line = tmp + 8;
-    tmp = strtok(line, "\n");
-    charset = strdup(tmp);
+    tmp += 8;
+    charset = strtok(tmp, "\n");
   }
   
   if (charset && (strncmp(charset, "UTF-8", 5) != 0)) {
-    cd = iconv_open("UTF-8", charset);
-    if (cd == (iconv_t)-1) {
-      perror("error in iconv_open");
-      return str;
-    }
-    inbuf = (char *) malloc(strlen(str) + 1);
-    if (inbuf)
-      strcpy(inbuf, str);
-    outbuf = malloc(outbufsize);
-    inchar = inbuf;
-    outchar = outbuf;
-    inbytesleft = strlen(inbuf);
-    outbytesleft = outbufsize;
-    ret_val = iconv(cd, &inchar, &inbytesleft, &outchar, &outbytesleft);
+    CFStringRef name;
+    CFStringEncoding cfencoding;
+    UInt32 nsencoding;
+    NSData *data;
 
-    if (ret_val == (size_t)-1 && errno != E2BIG) {
-      perror("error in iconv");
-      return str;
-    }
-    iconv_close(cd);
+    name = CFStringCreateWithCString(NULL, charset, kCFStringEncodingMacRoman);
+    cfencoding = CFStringConvertIANACharSetNameToEncoding(name);
+    CFRelease(name);
+    nsencoding = CFStringConvertEncodingToNSStringEncoding(cfencoding);
 
-    convstr = malloc(outbufsize - outbytesleft + 1);
-    strlcpy(convstr, outbuf, outbufsize - outbytesleft + 1);
-    free(outbuf);
-    free(inbuf);
-    free(charset);
-    str = convstr;
+    data = [NSData dataWithBytes:str length:strlen(str)];
+    convstr = [[NSString alloc] initWithData:data encoding:nsencoding];
+  } else {
+    convstr = [[NSString alloc] initWithUTF8String:str];
   }
-  return str;
+  free(line);
+
+  return convstr;
 }
