@@ -53,7 +53,9 @@ extern CFRunLoopSourceRef gUimRun;
 extern CFSocketContext gSockContext;
 
 
-//#define DEBUG_HELPER
+//#define DEBUG_HELPER  1
+//#define DEBUG_PREEDIT  1
+//#define DEBUG_CANDIDATES  1
 
 
 static void
@@ -71,7 +73,9 @@ ParseHelperString(const char *str);
 void
 UIMCommitString(void *ptr, const char *str)
 {
+#ifdef DEBUG_PREEDIT
   DEBUG_PRINT("UIMCommitString() len=%d\n", strlen(str));
+#endif
 
   //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
   MUIMSessionHandle handle = gActiveSession;
@@ -103,7 +107,9 @@ UIMPreeditClear(void *ptr)
   MUIMSessionHandle handle = gActiveSession;
   UInt32 i;
 
+#ifdef DEBUG_PREEDIT
   DEBUG_PRINT("UIMPreeditClear()\n");
+#endif
 
   for (i = 0; i < (*handle)->fSegmentCount; i++)
     free((*handle)->fSegments[i].fBuffer);
@@ -114,7 +120,7 @@ UIMPreeditClear(void *ptr)
 }
 
 void
-UIMPreeditPushback(void *ptr, int attr, char *str)
+UIMPreeditPushback(void *ptr, int attr, const char *str)
 {
   //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
   MUIMSessionHandle handle = gActiveSession;
@@ -145,9 +151,11 @@ UIMPreeditPushback(void *ptr, int attr, char *str)
       sprintf(attrstr, "%s %s", attrstr, tmpstr);
     }
 
+#ifdef DEBUG_PREEDIT
     DEBUG_PRINT("UIMPreeditPushback() fSegmentCount=%lu attr=%s len=%lu\n",
                 (*handle)->fSegmentCount,
                 attrstr, strlen(str));
+#endif
   }
 #endif
 
@@ -164,7 +172,9 @@ UIMPreeditUpdate(void *ptr)
   //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
   MUIMSessionHandle handle = gActiveSession;
 
+#ifdef DEBUG_PREEDIT
   DEBUG_PRINT("UIMPreeditUpdate()\n");
+#endif
 
   MUIMUpdateActiveInputArea(handle, FALSE);
 }
@@ -178,7 +188,9 @@ UIMCandAcivate(void *inPtr, int inNR, int inLimit)
   //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   MUIMSessionHandle handle = gActiveSession;
 
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandAcivate() inNR=%d inLimit=%d\n", inNR, inLimit);
+#endif
 
   InitCandidateWindow(handle);
 
@@ -200,7 +212,9 @@ UIMCandSelect(void *inPtr, int inIndex)
   //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   MUIMSessionHandle handle = gActiveSession;
 
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandSelect() inIndex=%d\n", inIndex);
+#endif
 
   (*handle)->fCandidateIndex = inIndex;
 
@@ -230,8 +244,10 @@ UIMCandShiftPage(void *inPtr, int inDirection)
       (*handle)->fCandidateIndex -= (*handle)->fDisplayLimit;
   }
 
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandShiftPage() inDirection=%d fCandidateIndex=%ld\n",
               inDirection, (*handle)->fCandidateIndex);
+#endif
 
   LayoutCandidate(handle);
   uim_set_candidate_index(gUC, (*handle)->fCandidateIndex);
@@ -246,7 +262,9 @@ UIMCandDeactivate(void *inPtr)
   //MUIMSessionHandle handle = (MUIMSessionHandle) inPtr;
   MUIMSessionHandle handle = gActiveSession;
 
+#ifdef DEBUG_CANDIDATES
   DEBUG_PRINT("UIMCandDeactivate()\n");
+#endif
 
   HideCandidateWindow(handle);
 }
@@ -304,7 +322,7 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
 }
 
 void
-UIMCheckHelper(MUIMSessionHandle inHandle)
+UIMCheckHelper()
 {
   if (gUimFD < 0) {
     gUimFD = uim_helper_init_client_fd(UIMHelperDisconnect);
@@ -316,7 +334,8 @@ UIMCheckHelper(MUIMSessionHandle inHandle)
     if (gUimFD >= 0) {
       if (!gUimSock) {
         gSockContext.version = 0;
-        gSockContext.info = inHandle;
+        //gSockContext.info = inHandle;
+        gSockContext.info = NULL;
         gSockContext.retain = NULL;
         gSockContext.release = NULL;
         gSockContext.copyDescription = NULL;
@@ -391,9 +410,11 @@ ParseHelperString(const char *str)
           (first = CFArrayGetValueAtIndex(array, 0))) {
         CFStringRef second = CFArrayGetValueAtIndex(array, 1);
         if (second) {
+#ifdef DEBUG_HELPER
           DEBUG_PRINT("ParseHelperString() first='%s' second='%s'\n",
                       CFStringGetCStringPtr(first, kCFStringEncodingUTF8),
                       CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
+#endif
           for (i = 0; i < gNumSession; i++) {
             if (*gSessionList[i])
               uim_prop_activate(gUC,
